<?php
/*--------------------------------------------------------------------
 OnGetProductModelEventInterface.php 2020-06-24
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -------------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Shop\SellingUnit\Database\Unit\Events\Interfaces;

use Gambio\Shop\Language\ValueObjects\LanguageId;
use Gambio\Shop\Product\ValueObjects\ProductId;
use Gambio\Shop\ProductModifiers\Modifiers\Collections\ModifierIdentifierCollectionInterface;
use Gambio\Shop\Properties\Properties\ValueObjects\CombinationId;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\Model;
use ProductDataInterface;
use Psr\EventDispatcher\EventDispatcherInterface;

/**
 * Interface OnGetProductModelEventInterface
 * @package Gambio\Shop\SellingUnit\Database\Unit\Events\Interfaces
 */
interface OnGetProductModelEventInterface
{
    
    
    /**
     * @return ProductDataInterface
     */
    public function product(): ProductDataInterface;
    
    
    /**
     * @return EventDispatcherInterface
     */
    public function dispatcher(): EventDispatcherInterface;
    
    
    /**
     * @return ProductId
     */
    public function productId(): ProductId;
    
    
    /**
     * @return ModifierIdentifierCollectionInterface
     */
    public function modifierIdentifiers(): ModifierIdentifierCollectionInterface;
    
    
    /**
     * @return LanguageId
     */
    public function languageId(): LanguageId;
    
    
    /**
     * @return Model
     */
    public function model(): Model;
    
    
    /**
     * @param Model $model
     */
    public function setModel(Model $model): void;


    /**
     * @return CombinationId|null
     */
    public function combinationId(): ?CombinationId;
}